@@ -146,7 +146,7 @@ The DONE message indicates that ghostscript has finished processing.
 #define GTK_GS_WATCH_INTERVAL 1000
 #define GTK_GS_WATCH_TIMEOUT  2
 
-enum { INTERPRETER_MESSAGE, INTERPRETER_ERROR, DOCUMENT_CHANGED, LAST_SIGNAL };
+enum { INTERPRETER_MESSAGE, INTERPRETER_ERROR, LAST_SIGNAL };
 
 static gboolean broken_pipe = FALSE;
 
@@ -347,7 +347,6 @@ gtk_gs_init (GtkGS *gs)
         gs->antialiased = gtk_gs_defaults_get_antialiased();
         gs->override_size = gtk_gs_defaults_get_override_size();
         gs->respect_eof = gtk_gs_defaults_get_respect_eof();
-        gs->watch_doc = gtk_gs_defaults_get_watch_doc();
         gs->show_scroll_rect = gtk_gs_defaults_get_show_scroll_rect();
         gs->scroll_step = gtk_gs_defaults_get_scroll_step();
         gs->zoom_mode = gtk_gs_defaults_get_zoom_mode();
@@ -391,17 +390,6 @@ gtk_gs_class_init (GtkGSClass *klass)
                        G_TYPE_NONE,
                        1, G_TYPE_INT);
 
-	gtk_gs_signals[DOCUMENT_CHANGED] = g_signal_new(
-                       "document_changed",
-                       G_TYPE_FROM_CLASS(object_class),
-                       G_SIGNAL_RUN_LAST,
-                       G_STRUCT_OFFSET(GtkGSClass, document_changed),
-                       NULL,
-                       NULL,
-                       g_cclosure_marshal_VOID__VOID,
-                       G_TYPE_NONE,
-                       0);
-
         object_class->destroy = gtk_gs_destroy;
 
         widget_class->realize       = gtk_gs_realize;
@@ -466,10 +454,6 @@ gtk_gs_cleanup (GtkGS *gs)
                 g_free(gs->gs_filename_unc);
                 gs->gs_filename_unc = NULL;
         }
-        if(gs->timer_tag) {
- 		gtk_timeout_remove (gs->timer_tag);
- 		gs->timer_tag = 0;
- 	}
         if(gs->pstarget && !gdk_window_is_visible(gs->pstarget))
                 gdk_window_hide(gs->pstarget);
         gs->current_page = -1;
@@ -682,9 +666,12 @@ gtk_gs_size_allocate (GtkWidget     *widget,
         /* 
          * update the adjustment if necessary (ie. a resize);
          */
+#if 0
         if (gs->zoom_mode != GTK_GS_ZOOM_ABSOLUTE) {
                 gtk_gs_set_zoom(gs, 0.0);
         }
+#endif
+
         if (GTK_WIDGET_REALIZED(gs)) {
                 gtk_gs_munge_adjustments (gs);
         }
@@ -798,32 +785,6 @@ send_ps(GtkGS *gs, long begin, unsigned int len, gboolean close)
         }
 }
 
-/* Reload file if it was changed */
-static gint
-timer_callback (gpointer data)
-{
-        GtkGS *gs;
-	struct stat sbuf;
-
-        g_assert (data);
-
-        gs = GTK_GS(data);
-
-        /* remove timer if no valid file is loaded */
-	if (!gs->loaded || !gs->watch_doc || !gs->gs_psfile) {
-                gs->timer_tag = 0;
-		return FALSE;
-	}
-
-	if (!gs->busy && !stat(gs->gs_filename, &sbuf) &&
-	    (sbuf.st_mtime != gs->mtime) &&
-	    (time (NULL) - sbuf.st_mtime >= GTK_GS_WATCH_TIMEOUT)) {
-                gtk_signal_emit(GTK_OBJECT(gs), gtk_gs_signals[DOCUMENT_CHANGED]);
-        }
-
-        return TRUE;
-}
-
 static void
 set_up_page (GtkGS *gs) 
      /* 
@@ -1241,7 +1202,7 @@ stop_interpreter (GtkGS *gs)
         if (gs->interpreter_pid > 0) {
                 int status = 0;
                 kill(gs->interpreter_pid, SIGTERM);
-                wait(&status);
+                while ((wait (&status) == -1) && (errno == EINTR)); 
 		gs->interpreter_pid = -1;
                 if(status == 1) {
                         gtk_gs_cleanup(gs);
@@ -1405,15 +1366,15 @@ check_pdf (GtkGS *gs)
                 }
                 close(fd);
 		fname = ggv_quote_filename (filename);
-		cmd = g_strdup_printf (gtk_gs_defaults_get_convert_pdf_cmd(),
+		cmd = g_strdup_printf (gtk_gs_defaults_get_dsc_cmd(),
                                        filename_dsc, fname);
 		g_free (fname);
 		/* this command (sometimes?) prints error messages to stdout! */
 		cmdline = g_strdup_printf ("%s >%s 2>&1", cmd, filename_err);
 		g_free (cmd);
 
-		if ((system (cmdline) == 0)
-		    && ggv_file_readable (filename_dsc) ) {
+		if ((system (cmdline) == 0) &&
+                    ggv_file_readable (filename_dsc) ) {
 
 			/* success */
 			filename = gs->gs_filename_dsc = filename_dsc;
@@ -1639,8 +1600,12 @@ gtk_gs_new (GtkAdjustment *hadj, GtkAdjustment *vadj)
 {
         GtkGS *gs;
 
-        g_return_val_if_fail(hadj != NULL, NULL);
-        g_return_val_if_fail(vadj != NULL, NULL);
+        if(NULL == hadj)
+                hadj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 1.0,
+                                                         0.01, 0.1, 0.09));
+        if(NULL == vadj)
+                vadj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 1.0,
+                                                         0.01, 0.1, 0.09));
 
 	gs = (GtkGS *)gtk_type_new(gtk_gs_get_type());
 
@@ -1742,8 +1707,6 @@ gtk_gs_disable_interpreter (GtkGS *gs)
 gboolean
 gtk_gs_load (GtkGS *gs, const gchar *fname)
 {
-        struct stat sbuf;
-
         g_return_val_if_fail(gs != NULL, FALSE);
         g_return_val_if_fail(GTK_IS_GS(gs), FALSE);
 
@@ -1840,20 +1803,11 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
                         /* Otherwise, set the proper orientation for the doc */
                         gs->real_orientation = gs->doc->orientation;
                 }
-                stat(fname, &sbuf);
-                gs->mtime = sbuf.st_mtime;
   	}
         gtk_gs_set_page_size(gs, -1, gs->current_page);
 	gtk_widget_queue_resize (&(gs->widget));
         gs->loaded = TRUE;
 
-	if(gs->watch_doc) {
-		if(gs->timer_tag)
-                        gtk_timeout_remove(gs->timer_tag);
-		gs->timer_tag = gtk_timeout_add(GTK_GS_WATCH_INTERVAL,
-                                                timer_callback, gs);
-	}
-
         gs->gs_status = _("Document loaded.");
 
 	return gs->loaded;
@@ -1929,14 +1883,6 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
                 return FALSE;
         }
 
-        if (gs->gs_psfile) {
-                /* has it changed? */
-                if (!stat(gs->gs_filename, &sbuf) &&
-                    gs->mtime != sbuf.st_mtime) {
-                        gtk_signal_emit(GTK_OBJECT(gs), gtk_gs_signals[DOCUMENT_CHANGED]);
-                }
-        }
-
         /* range checking... */
         if (page < 0)
                 page = 0;
@@ -2194,9 +2140,8 @@ gtk_gs_set_zoom (GtkGS *gs, gfloat zoom) {
                 zoom = ggv_zoom_levels[ggv_max_zoom_levels];
         if (fabs(gs->zoom_factor - zoom) > 0.001) { 
                 gs->zoom_factor = zoom;
-                if (GTK_WIDGET_REALIZED(gs)) {
+                if (GTK_WIDGET_REALIZED(gs)) 
                         set_up_page(gs);
-                }
                 gs->changed = TRUE;
                 gtk_widget_queue_resize (&(gs->widget));
         }
@@ -2216,15 +2161,19 @@ gtk_gs_zoom_to_fit(GtkGS *gs, gboolean fit_width)
 {
         gint new_y;
         gfloat new_zoom;
+        guint avail_w, avail_h;
 
         g_return_val_if_fail(gs != NULL, 0.0);
         g_return_val_if_fail(GTK_IS_GS(gs), 0.0);
 
-	new_zoom = (gfloat)gs->widget.allocation.width/(gfloat)gs->width*gs->zoom_factor;
+        avail_w = (gs->avail_w > 0)?gs->avail_w:gs->width;
+        avail_h = (gs->avail_h > 0)?gs->avail_h:gs->height;
+
+        new_zoom = ((gfloat)avail_w)/((gfloat)gs->width)*gs->zoom_factor;
 	if(!fit_width) {
-		new_y = new_zoom*gs->height/gs->zoom_factor;
-		if(new_y > gs->widget.allocation.height)
-			new_zoom = (gfloat)gs->widget.allocation.height/(gfloat)gs->height*gs->zoom_factor;
+		new_y = new_zoom*((gfloat)gs->height)/gs->zoom_factor;
+		if(new_y > avail_h)
+			new_zoom = ((gfloat)avail_h)/((gfloat)gs->height)*gs->zoom_factor;
 	}
 
         return new_zoom;
@@ -2356,33 +2305,6 @@ gtk_gs_get_antialiasing(GtkGS *gs)
         return gs->antialiased;
 }
 
-void
-gtk_gs_set_watch_file(GtkGS *gs, gboolean f)
-{
-        g_return_if_fail(gs != NULL);
-        g_return_if_fail(GTK_IS_GS(gs));
-
-        if(gs->watch_doc == f)
-                return;
-
-        gs->watch_doc = f;
-	if(gs->watch_doc) {
-		if(gs->timer_tag)
-                        gtk_timeout_remove(gs->timer_tag);
-		gs->timer_tag = gtk_timeout_add(GTK_GS_WATCH_INTERVAL,
-                                                timer_callback, gs);
-	}
-}
-
-gint
-gtk_gs_get_watch_file(GtkGS *gs)
-{
-        g_return_val_if_fail(gs != NULL, -1);
-        g_return_val_if_fail(GTK_IS_GS(gs), -1);
-
-        return gs->watch_doc;
-}
-
 const gchar *
 gtk_gs_get_document_title(GtkGS *gs)
 {
@@ -2457,6 +2379,15 @@ gtk_gs_start_scroll(GtkGS *gs)
                 gdk_draw_rectangle(gs->bpixmap, gs->psgc, FALSE,
                                    gs->scroll_start_x, gs->scroll_start_y,
                                    gs->scroll_width, gs->scroll_height);
+                rect.width = 1;
+                gdk_window_invalidate_rect(gs->pstarget, &rect, TRUE);
+                rect.x = gs->scroll_start_x + gs->scroll_width;
+                gdk_window_invalidate_rect(gs->pstarget, &rect, TRUE);
+                rect.x = gs->scroll_start_x + 1;
+                rect.width = gs->scroll_start_x + gs->scroll_width - 1;
+                rect.height = 1;
+                gdk_window_invalidate_rect(gs->pstarget, &rect, TRUE);
+                rect.y = gs->scroll_start_y + gs->scroll_height;
                 gdk_window_invalidate_rect(gs->pstarget, &rect, TRUE);
         }
 }
@@ -2479,6 +2410,15 @@ gtk_gs_end_scroll(GtkGS *gs)
                 gdk_draw_rectangle(gs->bpixmap, gs->psgc, FALSE,
                                    gs->scroll_start_x, gs->scroll_start_y,
                                    gs->scroll_width, gs->scroll_height);
+                rect.width = 1;
+                gdk_window_invalidate_rect(gs->pstarget, &rect, TRUE);
+                rect.x = gs->scroll_start_x + gs->scroll_width;
+                gdk_window_invalidate_rect(gs->pstarget, &rect, TRUE);
+                rect.x = gs->scroll_start_x + 1;
+                rect.width = gs->scroll_start_x + gs->scroll_width - 1;
+                rect.height = 1;
+                gdk_window_invalidate_rect(gs->pstarget, &rect, TRUE);
+                rect.y = gs->scroll_start_y + gs->scroll_height;
                 gdk_window_invalidate_rect(gs->pstarget, &rect, TRUE);
         }
         gs->scroll_start_x = -1;
@@ -2670,15 +2610,33 @@ gtk_gs_get_postscript(GtkGS *gs, gint *pages)
         sink = gtk_gs_doc_sink_new();
 
         if(GTK_GS_IS_PDF(gs)) {
-                /* Save marked pages of the dsc file. The saved
-                 * file is useless without the pdf file, but can
-                 * be used to print the marked pages.
-                 * Printing only works with gs, not with a 'real'
-                 * postscript printer.
-                 * FIXME: Can we produce a real pdf file with ps2pdf?
-                 */
-                pscopydoc(sink, gs->gs_filename_dsc,
-                          gs->doc, pages);
+                gchar *tmpn = g_strconcat(g_get_tmp_dir(), "/ggvXXXXXX", NULL);
+                gchar *cmd, *fname;
+                int tmpfd;
+
+                if((tmpfd = mkstemp(tmpn)) < 0) {
+                        g_free(tmpn);
+                        return NULL;
+                }
+                close(tmpfd);
+                fname = ggv_quote_filename(gs->gs_filename_unc?
+                                           gs->gs_filename_unc:
+                                           gs->gs_filename);
+                cmd = g_strdup_printf(gtk_gs_defaults_get_convert_pdf_cmd(),
+                                      tmpn, fname);
+                g_free(fname);
+                if((system(cmd) == 0) && ggv_file_readable(tmpn)) {
+                        GtkWidget *tmp_gs;
+                        tmp_gs = gtk_gs_new_from_file(NULL, NULL, tmpn);
+                        if(NULL != tmp_gs) {
+                                if(GTK_GS(tmp_gs)->loaded)
+                                        pscopydoc(sink, tmpn,
+                                                  GTK_GS(tmp_gs)->doc, pages);
+                                gtk_widget_destroy(tmp_gs);
+                        }
+                }
+                g_free(cmd);
+                g_free(tmpn);
         }
         else {
                 /* Use uncompressed file if necessary */
@@ -2686,7 +2644,6 @@ gtk_gs_get_postscript(GtkGS *gs, gint *pages)
         }
         if(free_pages)
                 g_free(pages);
-
         doc = gtk_gs_doc_sink_get_buffer(sink);
         gtk_gs_doc_sink_free(sink);
         return doc;
@@ -2779,3 +2736,13 @@ gtk_gs_get_zoom_mode(GtkGS *gs)
 {
         return gs->zoom_mode;
 }
+
+void
+gtk_gs_set_available_size(GtkGS *gs, guint avail_w, guint avail_h)
+{
+        gs->avail_w = avail_w;
+        gs->avail_h = avail_h;
+        if (gs->zoom_mode != GTK_GS_ZOOM_ABSOLUTE) {
+                gtk_gs_set_zoom(gs, 0.0);
+        }
+}
